function [F0Feature, F0Location, F0Track] = CalculateF0Features_MFCC(SpecData, EstF0, para)
% [F0Feature, F0Location, F0Track] = CalculateF0Features_MFCC(SpecData, EstF0, para)
% Calculate F0 feature vectors
% Input
%   - SpecData      : Complex spectrogram
%   - EstF0         : Multiple F0 estimation results, each column is a frame  
%   - para
%       - fs        : Sampling rate
%       - frameLen  : frame length
%       - fftLen    : fft length
%       - maxHarm   : The number of harmonics, i.e. harmonic structure feature dimensionality
%       - overlapTh : frequency deviation threshold of two harmonics to be judged being non-overlapped.
%       - trackNum  : The number of all sources in the file
%       - cepsOrder : Cepstrum order, i.e. the cepstrum feature dimentionality
% Output
%   - F0Feature     : Each row is a feature vector corresponds to a F0
%   - F0Location
%       - Col 1     : Frame number
%       - Col 2     : Frequency (MIDI number)
%   - F0Track       : F0 track label
%
% Author: Zhiyao Duan
% Created: 7/18/2012
% Last modified: 7/18/2012

fprintf('Calculating MFCC from the mixture signal...');

% parameters
fs = para.fs;
frameLen = para.frameLen;
fftLen = para.fftLen;
maxHarm = para.maxHarm;
harmOvlpTh = para.harmOvlpTh;
cepsOrder = para.cepsOrder;
trackNum = para.trackNum;

[MaxPoly, FrameNum] = size(EstF0);

F0Features = zeros(MaxPoly*FrameNum, cepsOrder);                    % feature vectors
F0Locations = zeros(MaxPoly*FrameNum, 3);                           % F0 time-frequency locations
F0Tracks = zeros(MaxPoly*FrameNum, 1);                              % Initial F0 tracks

% For each frame, we calculate the MFCC feature of each pitch (if exists)
% from the separated source signal of that pitch.
feaNum = 0;                                                         % count the number of feature vectors
for i = 1:FrameNum
    if mod(i, 10) == 0
        fprintf('.');
    end
    if mod(i, 1000) == 0
        fprintf('\n');
    end
    
    if sum(EstF0(:,i)~=0) == 0                                      % this frame is silent
        continue;
    end    
        
    % Separate the mixture
    para= {};
    para.fftLen = fftLen;
    para.fs = fs;
    para.harmOvlpTh = harmOvlpTh;                                 
    SepWavFrame = SeperateAFrame_Simple(SpecData(:,i), EstF0(:,i), trackNum, para);
%     para.maxHarm = maxHarm;
%     SepWavFrame = SeperateAFrame(SpecData(:,i), EstF0(:,i), trackNum, para);
    SepWavFrame = SepWavFrame(1:frameLen, :);
    
    for j = 1:MaxPoly
        if EstF0(j, i) == 0
            continue;
        end
        
        % Calculate MFCC for each source
        [ceps,aspectrum,pspectrum] = melfcc(SepWavFrame(:,j), fs, 'wintime', frameLen/fs, 'numcep', cepsOrder);
%         ceps = CalculateMFCC(SepWavFrame(:,j), fs, cepsOrder, 0);

        % compose the feature vector for each F0
        feaNum = feaNum + 1;
        F0Features(feaNum, :) = ceps;
        F0Locations(feaNum, 1) = i;
        F0Locations(feaNum, 2) = EstF0(j, i);
        F0Locations(feaNum, 3) = ceps(1);
        F0Tracks(feaNum) = j;                                      % An initial track label for each F0
    end
end

% We collected feaNum features throughout the file
F0Feature = F0Features(1:feaNum, :);
F0Location = F0Locations(1:feaNum, :);
F0Track = F0Tracks(1:feaNum);

fprintf('\n');
