function [F0Feature, F0Location, F0Track, F0Harmonics] = CalculateF0Features_UniformDiscreteCepstrum(SpecData, PeakData, PeakAmpData, EstF0, para)
% Calculate uniform discrete cepstrum feature vectors for estimated F0s
% Input
%   - PeakData      : Frequency (bin numbers) of extracted peaks of the audio file, each column is a frame
%   - PeakAmpData   : Amplitudes (dB) of extracted peaks of the audio file, each column is a frame
%   - EstF0         : Multiple F0 estimation results, each column is a frame  
%   - para
%       - fs        : Sampling rate
%       - fftLen    : FFT length
%       - maxHarm   : The number of harmonics, i.e. harmonic structure feature dimensionality
%       - Thresh_PeakF0Belong   : The threshold (in MIDI number) in frequency deviation to decide which peak belongs to which F0
%       - cepsOrder : Cepstrum order, i.e. the cepstrum feature dimentionality - 1
% Output
%   - F0Feature     : Each row is a feature vector corresponds to a F0
%   - F0Location
%       - Col 1     : Frame number
%       - Col 2     : Frequency (MIDI number)
%       - Col 3     : Energy of its harmonics (20 * log10 (sum linear amplitudes))
%   - F0Track       : F0 track label
%   - F0Harmonics   : Harmonic amplitudes (dB) of each F0
%
% Author: Zhiyao Duan
% Created: 7/19/2012
% Last modified: 7/19/2012

fprintf('Calculating cepstrum coefficient vectors...');

% parameters
fs = para.fs;
fftLen = para.fftLen;
maxHarm = para.maxHarm;
Thresh_PeakF0Belong = para.Thresh_PeakF0Belong;
cepsOrder = para.cepsOrder;

[MaxPoly, FrameNum] = size(EstF0);

% Initialize several variables
F0Features = zeros(MaxPoly*FrameNum, cepsOrder+1);                  % feature vectors
F0Locations = zeros(MaxPoly*FrameNum, 3);                           % F0 time-frequency locations
F0Tracks = zeros(MaxPoly*FrameNum, 1);                              % initial F0 tracks
F0Harmonics = zeros(MaxPoly*FrameNum, maxHarm);                     % harmonic amplitudes

% Start processing each frame
feaNum = 0;                                                         % count the number of feature vectors
for i = 1:FrameNum
    if mod(i, 10) == 0
        fprintf('.');
    end
    if mod(i, 1000) == 0
        fprintf('\n');
    end
    
    % find out peaks, might be empty
    peak = PeakData(:, i);
    peakAmp = PeakAmpData(:, i);
    
    idx = peak~=0;                                                  % because the peak number in each frame might be less than 100, so there are some 0s
    peak = peak(idx);
    peakAmp = peakAmp(idx);    
       
    mpeak = peak;                                                   % peak frequency in MIDI number
    fpeak = midi2hz(mpeak);                                         % peak frequency in Hz
    
    for j = 1:MaxPoly
        if EstF0(j, i) == 0                                        
            continue;
        end        
        
        mF0 = EstF0(j, i);                                          % F0 in MIDI number
        fF0 = midi2hz(mF0);                                         % F0 in Hz
    
        Ratio = fpeak / fF0;                                        % calculate the harmonic number of peak i given F01
        RRatio = round(Ratio);
      
        % Use frequency deviation to decide which peak can belong to which F0
        if isempty(mpeak)
            err = [];
        else
            err = mpeak - mF0 - 12*log2(max(RRatio,1));             % the frequency deviation of all peaks from harmonics of F01
        end
        
        % Calculate harmonics for each F0
        currMaxHarm = min(maxHarm, floor(fs/2/fF0));                % the maximum possible harmonic of this F0
        harmFreq = (1:currMaxHarm) * fF0;                           % harmonic frequencies
        harmAmp = zeros(1, currMaxHarm);                            % harmonic amplitudes
        for k = 1:currMaxHarm
            harmBin = round(fftLen*k*fF0/fs);                       % the frequency bin of this harmonic
            harmidx = find((abs(err)<Thresh_PeakF0Belong & (RRatio == k)) == 1);    % find all peaks that could belong to this harmonic
            if isempty(harmidx)
                harmAmp(k) = 20*log10(abs(SpecData(harmBin, i)));   % if no peak is associated, use the amplitude of the mixture spectrum
            else
                [~, idx] = min(abs(err(harmidx)));
                harmAmp(k) = peakAmp(harmidx(idx));                 % if there are multiple peaks, use the closest one
            end
        end
        idx = harmAmp~=0;
        
        % calculate the discrete cepstrum
        ceps = unidisceps(harmFreq(idx)/fs, harmAmp(idx), cepsOrder);
        
        % compose the feature vector for each F0
        feaNum = feaNum + 1;
        F0Features(feaNum, :) = ceps;        
        F0Locations(feaNum, 1) = i;
        F0Locations(feaNum, 2) = EstF0(j, i);
        F0Locations(feaNum, 3) = ceps(1);
        F0Tracks(feaNum) = j;                                      % An initial track label for each F0
        F0Harmonics(feaNum, 1:currMaxHarm) = harmAmp(1:currMaxHarm);
    end
end

% We have collected feaNum feature vectors
F0Feature = F0Features(1:feaNum, :);
F0Location = F0Locations(1:feaNum, :);
F0Track = F0Tracks(1:feaNum);
F0Harmonics = F0Harmonics(1:feaNum, :);

fprintf('\n');
